
# THIS FILE IS GENERATED FROM PYRAD_PROC SETUP.PY
short_version = '1.1.2'
version = '1.1.2'
full_version = '1.1.2'
git_revision = '1adcbe285823ae3d8d718aab42126087dc6d3f98'
compile_date_time = '2022-05-04 10:41'
username = 'wolfensb'
release = True

if not release:
    version = full_version
